LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;


ENTITY clock IS PORT(
	clk		: IN std_logic;
	clkClock: IN std_logic;
	set		: IN std_logic;
	inc		: IN std_logic;
	reset	: IN std_logic;

	show	: BUFFER std_logic_vector (2 DOWNTO 0);
	hours	: BUFFER std_logic_vector (3 DOWNTO 0);
	minutes	: BUFFER std_logic_vector (7 DOWNTO 0);
	seconds	: BUFFER std_logic_vector (7 DOWNTO 0));
END clock ;


ARCHITECTURE main OF clock IS
   SIGNAL mode: std_logic; -- 'Set' mode if == '1'
   SIGNAL modeTriggered: std_logic;
   SIGNAL digitSelection: integer RANGE 2 DOWNTO 0;
   SIGNAL modeCount: integer RANGE 50350000 DOWNTO 0;
   SIGNAL flashCount: integer RANGE 8391666 DOWNTO 0;
   SIGNAL flashState: std_logic;

BEGIN
   flasher: process(clk)
   begin
		if (clk'event and clk = '1') then
			if (reset = '0') then
				flashState <= '0';
				flashCount <= 0;
				show <= "000";
			else
				if (mode = '1') then
					if (flashCount = 8391666) then
						flashState <= not flashState;
						flashCount <= 0;
					else
						flashCount <= flashCount + 1;
					end if;
	
					if (digitSelection = 0) then
						show <= "00" & flashState;
					elsif (digitSelection = 1) then
						show <= "0" & flashState & "0";
					else
						show <= flashState & "00";
					end if;
				else
					show <= "000";
				end if;
			end if;
		end if;
	end process;

	modeCounter: process(clk)
		variable isSetting: boolean;

	begin
		if (clk'event and clk = '1') then
			if (reset = '0') then
				isSetting := false;
				modeCount <= 0;
				digitSelection <= 0;
				mode <= '0';
			else
				if (set = '0') then
					if (modeCount /= 50350000) then
						modeCount <= modeCount + 1;
						isSetting := true;
					end if;
	
					if (modeCount = 50349999) then
						isSetting := false;
						mode <= not mode;
						digitSelection <= 0;
					end if;
				else
					if (isSetting = true) then
						if (digitSelection = 2) then
							digitSelection <= 0;
						else
							digitSelection <= digitSelection + 1;
						end if;
	
						isSetting := false;
					end if;
	
					modeCount <= 0;
				end if;
			end if;
		end if;
	end process;

	clocker: process(clk)
		variable oldClkClock: std_logic;
		variable oldInc: std_logic;
		variable incCounter: integer RANGE 50350000 DOWNTO 0;

	begin
		if (clk'event and clk = '0') then
			if (reset = '0') then
				oldClkClock := '0';
				oldInc := '0';
				incCounter := 0;
				seconds <= "00000000";
				minutes <= "00000000";
				hours <= "0000";
			else
				if (mode = '1') then
					if (inc = '0') then
						if ((oldInc /= inc) or (incCounter = 50349999)) then -- Detect button activiation or extended hold down
							if (digitSelection = 0) then
								if (seconds(3 downto 0) = "1001") then
									if (seconds(7 downto 4) = "1001") then
										seconds(7 downto 4) <= "0000";
									else
										seconds(7 downto 4) <= signed(seconds(7 downto 4)) + 1;
									end if;
	
									seconds(3 downto 0) <= "0000";
								else
									seconds(3 downto 0) <= signed(seconds(3 downto 0)) + 1;
								end if;
							elsif (digitSelection = 1) then
								if (minutes(3 downto 0) = "1001") then
									if (minutes(7 downto 4) = "1001") then
										minutes(7 downto 4) <= "0000";
									else
										minutes(7 downto 4) <= signed(minutes(7 downto 4)) + 1;
									end if;
	
									minutes(3 downto 0) <= "0000";
								else
									minutes(3 downto 0) <= signed(minutes(3 downto 0)) + 1;
								end if;
							else
								if (hours(3 downto 0) = "1001") then
	 								hours(3 downto 0) <= "0000";
								else
									hours(3 downto 0) <= signed(hours(3 downto 0)) + 1;
								end if;
							end if;
						else
							if (incCounter = 50350000) then
								incCounter := 47203125;--44056250;
							end if;
						end if;
	
						incCounter := incCounter + 1;
					else
						incCounter := 0;
					end if;
					
					oldInc := inc;
				else
					if (oldClkClock /= clkClock) then
						if (clkClock = '1') then
							if (seconds(3 downto 0) = "1001") then
								if (seconds(7 downto 4) = "1001") then
									if (minutes(3 downto 0) = "1001") then
										if (minutes(7 downto 4) = "1001") then
											if (hours(3 downto 0) = "1001") then
												hours(3 downto 0) <= "0000";
											else
												hours(3 downto 0) <= signed(hours(3 downto 0)) + 1;
											end if;
	                     
											minutes(7 downto 4) <= "0000";
										else
											minutes(7 downto 4) <= signed(minutes(7 downto 4)) + 1;
										end if;
	                  
										minutes(3 downto 0) <= "0000";
									else
										minutes(3 downto 0) <= signed(minutes(3 downto 0)) + 1;
									end if;
	               
									seconds(7 downto 4) <= "0000";
								else
									seconds(7 downto 4) <= signed(seconds(7 downto 4)) + 1;
								end if;
	            
								seconds(3 downto 0) <= "0000";
							else
								seconds(3 downto 0) <= signed(seconds(3 downto 0)) + 1;
							end if;
						end if;
	
						oldClkClock := clkClock;
					end if;
				end if;
			end if;
		end if;
	end process;
END main;
