LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;


ENTITY decimalClock IS PORT( 
	clk				: IN std_logic;

	buttons			: IN std_logic_vector (1 downto 0);
	clockMultiplier: IN std_logic_vector (1 downto 0);
	studentSwitch	: IN std_logic;
	reset				: IN std_logic;

	enable			: OUT std_logic_vector (5 DOWNTO 0);
	segDisplay		: OUT std_logic_vector (7 DOWNTO 0);
	strobe			: OUT std_logic_vector (5 DOWNTO 0);

	studentDisplayH: OUT std_logic_vector (7 DOWNTO 0);
	studentDisplayL: OUT std_logic_vector (7 DOWNTO 0));
END decimalClock ;


ARCHITECTURE main OF decimalClock IS
   -- Internal signal declarations
   SIGNAL clkClock   : std_logic;
   SIGNAL clkDisplay : std_logic;
   SIGNAL hours      : std_logic_vector(3 DOWNTO 0);
   SIGNAL minutes    : std_logic_vector(7 DOWNTO 0);
   SIGNAL seconds    : std_logic_vector(7 DOWNTO 0);
   SIGNAL modeButton : std_logic;
   SIGNAL advanceButton : std_logic;
   SIGNAL studentNo  : std_logic_vector(7 downto 0);
   SIGNAL show       : std_logic_vector (2 DOWNTO 0);

   -- Component Declarations
   COMPONENT segDecoder PORT ( 
      value      : IN     std_logic_vector (3 DOWNTO 0);
      segDisplay : OUT    std_logic_vector (7 DOWNTO 0));
   END COMPONENT;
   COMPONENT debouncer PORT ( 
      clk        : IN     std_logic;
	  input		 : IN 	  std_logic;
      output     : BUFFER std_logic);
   END COMPONENT;
   COMPONENT clock PORT (
	  clk	   : IN		std_logic;
      clkClock : IN     std_logic ;
	  set	   : IN 	std_logic;
	  inc	   : IN		std_logic;
		reset : IN std_logic;
	  show	   : OUT    std_logic_vector (2 DOWNTO 0);
      hours    : BUFFER std_logic_vector (3 DOWNTO 0);
      minutes  : BUFFER std_logic_vector (7 DOWNTO 0);
      seconds  : BUFFER std_logic_vector (7 DOWNTO 0));
   END COMPONENT;
   COMPONENT clockDivider PORT (
      clk        : IN     std_logic ;
	  clockMultiplier: IN std_logic_vector (1 downto 0);
      clkClock   : BUFFER std_logic ;
      clkDisplay : BUFFER std_logic);
   END COMPONENT;
   COMPONENT displayDriver PORT (
      clkDisplay : IN     std_logic ;
      hours      : IN     std_logic_vector (3 DOWNTO 0);
      minutes    : IN     std_logic_vector (7 DOWNTO 0);
      seconds    : IN     std_logic_vector (7 DOWNTO 0);
	  show		 : IN	  std_logic_vector (2 DOWNTO 0);
      enable     : OUT    std_logic_vector (5 DOWNTO 0);
      segDisplay : OUT    std_logic_vector (7 DOWNTO 0);
      strobe     : OUT    std_logic_vector (5 DOWNTO 0));
   END COMPONENT;

   --FOR ALL : clock USE ENTITY decimalClock_lib.clock;
   --FOR ALL : clockDivider USE ENTITY decimalClock_lib.clockDivider;
   --FOR ALL : displayDriver USE ENTITY decimalClock_lib.displayDriver;

BEGIN
   with studentSwitch select
		studentNo <= "10010011" when '1',
				     "10000011" when others;

   -- Instance port mappings.
   I5 : segDecoder PORT MAP (
		 value => studentNo(7 downto 4),
		 segDisplay => studentDisplayH);
   I6 : segDecoder PORT MAP (
		 value => studentNo(3 downto 0),
		 segDisplay => studentDisplayL);

   I3 : debouncer PORT MAP (
		 clk => clk,
		 input => buttons(1),
		 output => modeButton);
   I4 : debouncer PORT MAP (
		 clk => clk,
		 input => buttons(0),
		 output => advanceButton);

   I1 : clock PORT MAP (
		 clk => clk,
         clkClock => clkClock,
		 set => modeButton,
		 inc => advanceButton,
		reset => reset,
		 show => show,
         hours    => hours,
         minutes  => minutes,
         seconds  => seconds);

   I0 : clockDivider PORT MAP (
         clk        => clk,
		 clockMultiplier => clockMultiplier,
         clkClock   => clkClock,
         clkDisplay => clkDisplay);
   I2 : displayDriver PORT MAP (
         clkDisplay => clkDisplay,
         hours      => hours,
         minutes    => minutes,
         seconds    => seconds,
		 show		=> show,
         enable     => enable,
         segDisplay => segDisplay,
         strobe     => strobe);
END main;
